<?php

namespace Intergo\SubscriptionClient;

use Intergo\BaseClient\Enums\HttpMethodEnum;
use Intergo\BaseClient\Responses\BaseResponseInterface;
use Intergo\BaseClient\Responses\ErrorResponse;
use Psr\Http\Message\ResponseInterface;

abstract class AbstractSubscriptionService
{
    private SubscriptionClient $client;
    protected string $secret;
    protected string $site;

    const LOGIN_RESULT_TOKEN = 'token';
    const LOGIN_RESULT_USER = 'user';


    /**
     * AbstractSubscriptionService constructor.
     * https://intergo.atlassian.net/wiki/spaces/ES/pages/2149187587/Communication+between+microservices
     *
     * @param $site
     * @param $baseUrl
     * @param $secret
     * @param $traceId
     */
    public function __construct($site, $baseUrl, $secret, $traceId)
    {
        $this->client = new SubscriptionClient($baseUrl, $traceId);
        $this->secret = $secret;
        $this->site = $site;
    }

    /**
     * @param $params
     * @return array
     */
    protected function setParams($params)
    {
        return array_merge([
            'site' => $this->site,
            'jwt_auth_secret' => $this->secret,
        ], $params);
    }

    /**
     * @param $endpoint
     * @param $bodyParams
     * @param array $urlParams
     * @return BaseResponseInterface
     */
    protected function execute($endpoint, $bodyParams, array $urlParams = [])
    {
        $bodyParams = $this->setParams($bodyParams);
        $email = $bodyParams['email'] ?? $bodyParams['username'] ?? '';
        $result = $this->client->execute($endpoint, $urlParams, $bodyParams);
        return $this->response($endpoint, $email, $bodyParams['ip'] ?? '', $result);
    }

    /**
     * @param string $action
     * @param $email
     * @param $ip
     * @param BaseResponseInterface $result
     * @return BaseResponseInterface
     */
    abstract public function response(string $action, $email, $ip, BaseResponseInterface $result);

    abstract public function storeJWT($authToken, $authUser);


    /** ################################################ AUTH ACTIONS ################################################ */

    /**
     * @return BaseResponseInterface
     */
    public function getAllPlans()
    {
        return $this->execute(__FUNCTION__, []);
    }

    /**
     * @param $ip
     * @return BaseResponseInterface
     */
    public function getPlanById($id): BaseResponseInterface
    {
        return $this->execute(__FUNCTION__, [],compact('id'));
    }

    /**
     * @param $attributes
     * @return BaseResponseInterface
     */
    public function createPlan($attributes): BaseResponseInterface
    {
        return $this->execute(__FUNCTION__, get_defined_vars());
    }

    /**
     * @param $id
     * @param $attributes
     * @return BaseResponseInterface
     */
    public function updatePlan($id, $attributes): BaseResponseInterface
    {
        return $this->execute(__FUNCTION__, $attributes,compact('id'));
    }

    /**
     * @param $ip
     * @return BaseResponseInterface
     */
    public function deletePlan($id): BaseResponseInterface
    {
        return $this->execute(__FUNCTION__, [],compact('id'));
    }
}